#import "BlueThermalPrinterPlugin.h"
#import <blue_thermal_printer/blue_thermal_printer-Swift.h>
#import "HLBLEManager.h"
#import "HLPrinter.h"
#import "TscCommand.h"

@interface BlueThermalPrinterPlugin ()

@property(nonatomic, retain) NSObject<FlutterPluginRegistrar> *registrar;
@property(nonatomic, retain) FlutterMethodChannel *channel;
@property(nonatomic, retain) BluetoothPrintStreamHandler *stateStreamHandler;
@property(nonatomic) NSMutableDictionary *scannedPeripherals;

@property (nonatomic, strong) HLBLEManager *manager;
@property (nonatomic, strong) NSMutableArray *deviceArray;  /**< 蓝牙设备>*/
@property (nonatomic, strong) NSString *lastDevice;
@property (nonatomic, assign) BOOL isConnected;
@property (assign, nonatomic) NSTimeInterval timeout;       /**< 默认超时时间 */

@end

@implementation BlueThermalPrinterPlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    //[SwiftBlueThermalPrinterPlugin registerWithRegistrar:registrar];
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:NAMESPACE @"/methods"
                                     binaryMessenger:[registrar messenger]];
    FlutterEventChannel* stateChannel = [FlutterEventChannel eventChannelWithName:NAMESPACE @"/state" binaryMessenger:[registrar messenger]];
    
    BlueThermalPrinterPlugin* instance = [[BlueThermalPrinterPlugin alloc] init];
    instance.channel = channel;
    instance.scannedPeripherals = [NSMutableDictionary new];
    
    // 初始化数据
    [instance setupData];
    // 蓝牙模块
    [instance initBluetoothToScan];
    
    // STATE
    BluetoothPrintStreamHandler* stateStreamHandler = [[BluetoothPrintStreamHandler alloc] init];
    [stateChannel setStreamHandler:stateStreamHandler];
    instance.stateStreamHandler = stateStreamHandler;
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSLog(@"call method -> %@", call.method);
    
    NSDictionary *arguments = call.arguments;
    __weak BlueThermalPrinterPlugin *weakSelf = self;
    if ([@"state" isEqualToString:call.method]) {
        result(nil);
    }
    else if([@"isAvailable" isEqualToString:call.method]) {
        result(@(YES));
    }
    else if([@"isOn" isEqualToString:call.method]) {
        result(@(YES));
    }
    else if([@"isConnected" isEqualToString:call.method]) {
        result(@(weakSelf.isConnected));
    }
    else if([@"getLastDevice" isEqualToString:call.method]) {
        NSString *lastDevice = [HLBLEManager UUIDStringForLastPeripheral];
        @try {
            result(lastDevice != nil ? lastDevice:@"");
        } @catch (FlutterError *e) {
            result(e);
        }
    }
    else if([@"openSettings" isEqualToString:call.method]) {
        
    }
    else if([@"getBondedDevices" isEqualToString:call.method]) {
        result(weakSelf.deviceArray);
    }
    else if([@"connect" isEqualToString:call.method]) {
        @try {
            if ([arguments.allKeys containsObject:@"address"]) {
                NSString *name = arguments[@"name"];
                weakSelf.lastDevice = name;
                NSLog(@"连接蓝牙名称：%@", name);
                CBPeripheral *peripheral = [_scannedPeripherals objectForKey:[arguments objectForKey:@"address"]];
                // 查询连接外围设备
                [weakSelf connectBLEDevice:peripheral];
            }
            result(nil);
        } @catch(FlutterError *e) {
            result(e);
        }
    }
    else if([@"disconnect" isEqualToString:call.method]) {
        @try {
            [weakSelf.manager cancelPeripheralConnection];
            weakSelf.isConnected = NO;
            result(nil);
        } @catch(FlutterError *e) {
            result(e);
        }
    }
    else if([@"printSampleCode" isEqualToString:call.method]) {
        @try {
            [weakSelf sendPrintDataEvent:arguments];
            result(nil);
        } @catch(FlutterError *e) {
            result(e);
        }
    }
    else if([@"repeatScan" isEqualToString:call.method]) {
        [weakSelf startScanPerpheralTimeout:10];
        result(nil);
    }
    else if([@"writeBytes" isEqualToString:call.method]) {
        @try {
            if ([arguments.allKeys containsObject:@"message"]) {
                FlutterStandardTypedData *bytesList = arguments[@"message"];
                [weakSelf writeBytes:bytesList.data];
            }
            result(nil);
        } @catch (FlutterError *e) {
            result(e);
        }
    }
    else if([@"getBarCode" isEqualToString:call.method]) {
        @try {
            if ([arguments.allKeys containsObject:@"textToBar"]) {
                [weakSelf getBarCode:result arguments:arguments];
            }
            result(nil);
        } @catch (FlutterError *e) {
            result(e);
        }
    }
    else if([@"printTSCImage" isEqualToString:call.method]) {
        @try {
            NSDictionary *args = [call arguments];
            [weakSelf writeBytes:[self mapToTscCommand:args]];
            result(nil);
        } @catch(FlutterError *e) {
            result(e);
        }
    }
}

#pragma mark - 打印 method

- (void)writeBytes:(NSData *)mainData {
    [self.manager sendPrintData:mainData completion:^(CBPeripheral *connectPerpheral, BOOL completion, NSString *error) {
        NSLog(@"写入结：%d---错误:%@",completion,error);
    }];
}

- (void)sendPrintDataEvent:(NSDictionary *)arguments {
    HLPrinter *printer = [self getPrinter:arguments];
    if (printer == nil) {
        return;
    }
    NSData *mainData = [printer getFinalData];
    [self.manager sendPrintData:mainData completion:^(CBPeripheral *connectPerpheral, BOOL completion, NSString *error) {
        NSLog(@"写入结：%d---错误:%@",completion,error);
    }];
}

- (HLPrinter *)getPrinter:(NSDictionary *)dict {
    HLPrinter *printer = [[HLPrinter alloc] init];
    NSString *qrcode = dict[@"qrcode"];
    NSString *barcode = dict[@"barcode"];
    NSString *middleTitle = dict[@"title"];             //小票title
    NSString *subTitle = dict[@"subTitle"];             //小票subTitle
    NSString *smallSubTitle = dict[@"smallSubTitle"];   //小票smallSubTitle
    NSString *footerInfo = dict[@"bottomContent"];      //小票底部文案
    
    if (qrcode.length <= 0 || barcode.length <= 0) { return nil; }
    /*
     * 制作小票代码
     */
    if (middleTitle.length > 0) {
        NSUInteger lengthT = [self stringEncode:middleTitle];
        if (lengthT <= 16) {
            if (lengthT % 2 == 0) { // 偶数
                NSInteger offset = ceil((16 - lengthT)/2) * (48/2);
                [printer appendTitle:@"" value:middleTitle valueOffset:offset fontSize:HLFontSizeTitleMiddle];
            } else {
                NSInteger offset = ceil((16 - lengthT)/2) * (54/2);
                [printer appendTitle:@"" value:middleTitle valueOffset:offset fontSize:HLFontSizeTitleMiddle];
            }
        } else {
            [printer appendTitle:@"" value:middleTitle valueOffset:0 fontSize:HLFontSizeTitleMiddle];
        }
    }
    if (subTitle.length > 0) {
        NSUInteger lengthT = [self stringEncode:subTitle];
        if (lengthT <= 32) {
            NSInteger offset = (32 - lengthT)/2 * (25/2);
            [printer appendTitle:@"" value:subTitle valueOffset:offset fontSize:HLFontSizeTitleSmalle];
        } else {
            [printer appendTitle:@"" value:subTitle valueOffset:0 fontSize:HLFontSizeTitleSmalle];
        }
    }
    if (smallSubTitle.length > 0) {
        NSUInteger lengthT2 = [self stringEncode:smallSubTitle];
        if (lengthT2 <= 32) {
            NSInteger offset = (32 - lengthT2)/2 * (25/2);
            [printer appendTitle:@"" value:smallSubTitle valueOffset:offset fontSize:HLFontSizeTitleSmalle];
        } else {
            [printer appendTitle:@"" value:smallSubTitle valueOffset:0 fontSize:HLFontSizeTitleSmalle];
        }
    }
    [printer appendQRCodeWithInfo:qrcode size:8 alignment:HLTextAlignmentLeft offset:90];
    [printer appendBarCodeWithInfo:barcode alignment:HLTextAlignmentLeft maxWidth:300];
    [printer appendFooter:footerInfo];
    
    return printer;
}

- (NSUInteger)stringEncode:(NSString *)s {
    NSStringEncoding enc = CFStringConvertEncodingToNSStringEncoding(kCFStringEncodingGB_18030_2000);
    NSData *data = [s dataUsingEncoding:enc];
    NSUInteger lengthT = [data length];
    return lengthT;
}

- (void)getBarCode:(FlutterResult)result arguments:(NSDictionary *)arguments {
    NSString *textToBar = arguments[@"textToBar"];
    NSInteger width = [arguments[@"width"] integerValue];
    NSInteger height = [arguments[@"height"] integerValue];
    NSInteger align = [arguments[@"align"] integerValue];

    HLTextAlignment alignment;
    switch (align) {
        case 0:
        {
            alignment = HLTextAlignmentLeft;
        } break;
        case 1:
        {
            alignment = HLTextAlignmentCenter;
        } break;
        case 2:
        {
            alignment = HLTextAlignmentRight;
        } break;
            
        default:
            alignment = HLTextAlignmentCenter;
            break;
    }
    
    if (textToBar.length <= 0) { result(nil); return; }
    HLPrinter *printer = [[HLPrinter alloc] init];
    [printer getBarCodeWithInfo:textToBar
                      alignment:alignment
                         offset:40
                          width:width
                         height:height];
    NSData *mainData = [printer getFinalData];
    FlutterStandardTypedData *dataByte = [FlutterStandardTypedData typedDataWithBytes:mainData];
    result(dataByte);
}

- (NSData *)mapToTscCommand:(NSDictionary *) args {
    NSDictionary *config = [args objectForKey:@"config"];
    NSMutableArray *list = [args objectForKey:@"data"];
    
    NSNumber *width = ![config objectForKey:@"width"]?@"48" : [config objectForKey:@"width"];
    NSNumber *height = ![config objectForKey:@"height"]?@"80" : [config objectForKey:@"height"];
    NSNumber *gap = ![config objectForKey:@"gap"]?@"2" : [config objectForKey:@"gap"];
    
    TscCommand *command = [[TscCommand alloc]init];
    // 设置标签尺寸宽高，按照实际尺寸设置 单位mm
    [command addSize:[width intValue] :[height intValue]];
    // 设置标签间隙，按照实际尺寸设置，如果为无间隙纸则设置为0 单位mm
    [command addGapWithM:[gap intValue] withN:0];
    // 设置原点坐标
    [command addReference:0 :0];
    // 撕纸模式开启
    [command addTear:@"ON"];
    // 开启带Response的打印，用于连续打印
    [command addQueryPrinterStatus:ON];
    // 清除打印缓冲区
    [command addCls];
    
    for(NSDictionary *m in list){
        
        NSString *type = [m objectForKey:@"type"];
        NSString *content = [m objectForKey:@"content"];
        NSNumber *x = ![m objectForKey:@"x"]?@0 : [m objectForKey:@"x"];
        NSNumber *y = ![m objectForKey:@"y"]?@0 : [m objectForKey:@"y"];
        NSNumber *imageWidth = ![m objectForKey:@"width"]?@300 : [m objectForKey:@"width"];
        
        if([@"text" isEqualToString:type]){
            [command addTextwithX:[x intValue] withY:[y intValue] withFont:@"TSS24.BF2" withRotation:0 withXscal:1 withYscal:1 withText:content];
        }else if([@"barcode" isEqualToString:type]){
            [command add1DBarcode:[x intValue] :[y intValue] :@"CODE128" :100 :1 :0 :2 :2 :content];
        }else if([@"qrcode" isEqualToString:type]){
            [command addQRCode:[x intValue] :[y intValue] :@"L" :5 :@"A" :0 :content];
        }else if([@"image" isEqualToString:type]){
            NSData *decodeData = [[NSData alloc] initWithBase64EncodedString:content options:0];
            UIImage *image = [UIImage imageWithData:decodeData];
            [command addBitmapwithX:[x intValue] withY:[y intValue] withMode:0 withWidth:[imageWidth intValue] withImage:image];
        }
        
    }
    
    [command addPrint:1 :1];
    
    return [command getCommand];
}

#pragma mark -
#pragma mark - setup 初始化数据

- (void)setupData {
    self.isConnected = NO;
    self.timeout = 30;
}

- (void)startScanPerpheralTimeout:(NSTimeInterval)timeout {
    // 清空字典
    [self.scannedPeripherals removeAllObjects];
    [self.deviceArray removeAllObjects];
    self.timeout = timeout;
    
    // 不重复扫描已发现设备
    // CBCentralManagerScanOptionAllowDuplicatesKey设置为NO表示不重复扫瞄已发现设备，为YES就是允许。
    // CBCentralManagerOptionShowPowerAlertKey设置为YES就是在蓝牙未打开的时候显示弹框
    NSDictionary *option = @{
        CBCentralManagerScanOptionAllowDuplicatesKey : [NSNumber numberWithBool:NO],
        CBCentralManagerOptionShowPowerAlertKey:[NSNumber numberWithBool:YES]};
    
    __weak BlueThermalPrinterPlugin *weakSelf = self;
    [self.manager scanForPeripheralsWithServiceUUIDs:nil options:option didDiscoverPeripheral:^(CBCentralManager *central, CBPeripheral *peripheral, NSDictionary *advertisementData, NSNumber *RSSI) {
        NSLog(@"扫描的外围设备名称 -> name： %@", peripheral.name);
        if (peripheral.name.length <= 0) { return ; }
        
        if (self.deviceArray.count == 0) {
            NSDictionary *dict = @{
                @"name":peripheral.name,
                @"address":peripheral.identifier.UUIDString,
            };
            [weakSelf.deviceArray addObject:dict];
        } else {
            BOOL isExist = NO;
            for (int i = 0; i < self.deviceArray.count; i++) {
                NSDictionary *dict = [weakSelf.deviceArray objectAtIndex:i];
                NSString *address = dict[@"address"];
                if ([address isEqualToString:peripheral.identifier.UUIDString]) {
                    isExist = YES;
                    NSDictionary *dict = @{
                        @"name":peripheral.name,
                        @"address":peripheral.identifier.UUIDString,
                    };
                    [weakSelf.deviceArray replaceObjectAtIndex:i withObject:dict];
                }
            }
            if (!isExist) {
                NSDictionary *dict = @{
                    @"name":peripheral.name,
                    @"address":peripheral.identifier.UUIDString,
                };
                [weakSelf.deviceArray addObject:dict];
            }
        }
        
        [weakSelf.scannedPeripherals setObject:peripheral forKey:[[peripheral identifier] UUIDString]];
        NSLog(@"扫描的蓝牙外设数量 -> %lu", (unsigned long)weakSelf.deviceArray.count);
    }];
}

// 开始扫描
- (void)initBluetoothToScan {
    __weak BlueThermalPrinterPlugin *weakSelf = self;
    self.manager.stateUpdateBlock = ^(CBCentralManager *central) {
        NSString *info = nil;
        switch (central.state) {
            case CBCentralManagerStatePoweredOn:{
                info = @"蓝牙已打开，并且可用";
                [weakSelf startScanPerpheralTimeout:10];
                break;
            }
            case CBCentralManagerStatePoweredOff:
                info = @"蓝牙可用，未打开";
                break;
            case CBCentralManagerStateUnsupported:
                info = @"SDK不支持";
                break;
            case CBCentralManagerStateUnauthorized:
                info = @"程序未授权";
                break;
            case CBCentralManagerStateResetting:
                info = @"CBCentralManagerStateResetting";
                break;
            case CBCentralManagerStateUnknown:
                info = @"CBCentralManagerStateUnknown";
                break;
        }
        NSLog(@"扫描中心设备状态 -> state：%@", info);
    };
}

// 连接外围设备
- (void)connectBLEDevice:(CBPeripheral *)perpheral {
    if (perpheral.name.length <= 0) {
        return;
    }
    
    //HLBLEManager *manager = [HLBLEManager sharedInstance];
    __weak BlueThermalPrinterPlugin *weakSelf = self;
    [self.manager connectPeripheral:perpheral
                     connectOptions:@{CBConnectPeripheralOptionNotifyOnDisconnectionKey:@(YES)}
             stopScanAfterConnected:YES
                    servicesOptions:nil
             characteristicsOptions:nil
                      completeBlock:^(HLOptionStage stage, CBPeripheral *peripheral, CBService *service, CBCharacteristic *character, NSError *error) {
        switch (stage) {
            case HLOptionStageConnection:
            {
                NSLog(@"连接的外设状态 -> state: %ld", (long)peripheral.state);
                if (error) {
                    NSLog(@"连接失败");
                    weakSelf.isConnected = NO;
                } else { // 状态变化
                    [weakSelf updateConnectState:peripheral.state];
                }
                break;
            }
            case HLOptionStageSeekServices:
            {
                if (error) {
                    NSLog(@"查找服务失败");
                } else {
                    NSLog(@"查找服务成功");
                }
                break;
            }
            case HLOptionStageSeekCharacteristics:
            {
                // 该block会返回多次，每一个服务返回一次
                if (error) {
                    NSLog(@"查找特性失败");
                } else {
                    NSLog(@"查找特性成功");
                }
                break;
            }
            case HLOptionStageSeekdescriptors:
            {
                // 该block会返回多次，每一个特性返回一次
                if (error) {
                    NSLog(@"查找特性的描述失败");
                } else {
                    NSLog(@"查找特性的描述成功");
                }
                break;
            }
            default:
                break;
        }
    }];
}

- (void)updateConnectState:(CBPeripheralState)state {
    __weak BlueThermalPrinterPlugin *weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        NSNumber *ret = @0;
        switch (state) {
            case CBPeripheralStateConnecting:
            {
                NSLog(@"update status -> %@", @"连接状态：连接中....");
                ret = @0;
                weakSelf.isConnected = NO;
                break;
            }
            case CBPeripheralStateConnected:
            {
                NSLog(@"update status -> %@", @"连接状态：连接成功");
                ret = @1;
                weakSelf.isConnected = YES;
                break;
            }
            case CBPeripheralStateDisconnecting:
            {
                NSLog(@"update status -> %@", @"连接状态：连接失败");
                ret = @0;
                weakSelf.isConnected = NO;
                break;
            }
            case CBPeripheralStateDisconnected:
            {
                NSLog(@"update status -> %@", @"连接状态：断开连接");
                ret = @0;
                weakSelf.isConnected = NO;
                break;
            }
            default:
                NSLog(@"update status -> %@", @"连接状态：连接超时");
                ret = @0;
                weakSelf.isConnected = NO;
                break;
        }
        
        NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:ret,@"id",nil];
        if(weakSelf.stateStreamHandler.sink != nil) {
            weakSelf.stateStreamHandler.sink([dict objectForKey:@"id"]);
        }
    });
}

#pragma mark -
#pragma mark - 懒加载

- (NSMutableArray *)deviceArray {
    if (!_deviceArray) {
        _deviceArray = [[NSMutableArray alloc] init];
    }
    return _deviceArray;
}

- (HLBLEManager *)manager {
    if (!_manager) {
        _manager = [HLBLEManager sharedInstance];
    }
    return _manager;
}

- (void)setTimeout:(NSTimeInterval)timeout {
    _timeout = timeout;
    if (_timeout > 0) {
        [self performSelector:@selector(timeoutAction) withObject:nil afterDelay:timeout];
    }
}

- (void)timeoutAction {
    [self.manager stopScan];
}

@end


@implementation BluetoothPrintStreamHandler

- (FlutterError*)onListenWithArguments:(id)arguments eventSink:(FlutterEventSink)eventSink {
    self.sink = eventSink;
    return nil;
}

- (FlutterError*)onCancelWithArguments:(id)arguments {
    self.sink = nil;
    return nil;
}

@end
